/*
 *                            Yeppp! library header
 *                   This file is auto-generated by Peach-Py,
 *        Portable Efficient Assembly Code-generator in Higher-level Python,
 *                    part of the Yeppp! library infrastrure
 * 
 * This file is part of Yeppp! library and licensed under the New BSD license.
 * 
 * Copyright (C) 2010-2012 Marat Dukhan
 * Copyright (C) 2012-2013 Georgia Institute of Technology
 * All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Georgia Institute of Technology nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <yepPredefines.h>
#include <yepTypes.h>

#ifdef __cplusplus
	extern "C" {
#endif

/** @defgroup yepCore yepCore.h: basic arithmetic operations. */

/**
 * @ingroup yepCore
 * @defgroup yepCore_Add	Addition
 */
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 8-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 8-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 8-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>ARM</td><td>ARM Cortex-A9</td><td>VFP2, NEON</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8uV8u_V8u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep8u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 8-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 8-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 16-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8uV8u_V16u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two signed 8-bit integer arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 8-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of signed 8-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of signed 16-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8sV8s_V16s(const Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, Yep16s *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 16-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 16-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 16-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>ARM</td><td>ARM Cortex-A9</td><td>VFP2, NEON</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16uV16u_V16u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 16-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 16-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 32-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16uV16u_V32u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two signed 16-bit integer arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 16-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of signed 16-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of signed 32-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16sV16s_V32s(const Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, Yep32s *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 32-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 32-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 32-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>ARM</td><td>ARM Cortex-A9</td><td>VFP2, NEON</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32uV32u_V32u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 32-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 32-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 64-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32uV32u_V64u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two signed 32-bit integer arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 32-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of signed 32-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of signed 64-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32sV32s_V64s(const Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep64s *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two unsigned 64-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 64-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 64-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 64-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V64uV64u_V64u(const Yep64u *YEP_RESTRICT x, const Yep64u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two single precision (32-bit) floating-point arrays, producing an array of single precision (32-bit) floating-point elements.
 * @param[in]	x	Pointer the first array of single precision (32-bit) floating-point elements to be added.
 * @param[in]	y	Pointer the second array of single precision (32-bit) floating-point elements to be added.
 * @param[out]	sum	Pointer the array of single precision (32-bit) floating-point elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32fV32f_V32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Add
 * @brief	Adds corresponding elements in two double precision (64-bit) floating-point arrays, producing an array of double precision (64-bit) floating-point elements.
 * @param[in]	x	Pointer the first array of double precision (64-bit) floating-point elements to be added.
 * @param[in]	y	Pointer the second array of double precision (64-bit) floating-point elements to be added.
 * @param[out]	sum	Pointer the array of double precision (64-bit) floating-point elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V64fV64f_V64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup yepCore
 * @defgroup yepCore_Subtract	Subtraction
 */
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 8-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 8-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 8-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8uV8u_V8u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep8u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 8-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 8-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 16-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8uV8u_V16u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two signed 8-bit integer arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	x	Pointer the minuend array of signed 8-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of signed 8-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of signed 16-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8sV8s_V16s(const Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, Yep16s *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 16-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 16-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 16-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16uV16u_V16u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 16-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 16-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 32-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16uV16u_V32u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two signed 16-bit integer arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	x	Pointer the minuend array of signed 16-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of signed 16-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of signed 32-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16sV16s_V32s(const Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, Yep32s *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 32-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 32-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 32-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32uV32u_V32u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 32-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 32-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 64-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32uV32u_V64u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two signed 32-bit integer arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	x	Pointer the minuend array of signed 32-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of signed 32-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of signed 64-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32sV32s_V64s(const Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep64s *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two unsigned 64-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 64-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 64-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 64-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V64uV64u_V64u(const Yep64u *YEP_RESTRICT x, const Yep64u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two single precision (32-bit) floating-point arrays, producing an array of single precision (32-bit) floating-point elements.
 * @param[in]	x	Pointer the minuend array of single precision (32-bit) floating-point elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of single precision (32-bit) floating-point elements to be subtracted.
 * @param[out]	difference	Pointer the array of single precision (32-bit) floating-point elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32fV32f_V32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore_Subtract
 * @brief	Subtracts corresponding elements in two double precision (64-bit) floating-point arrays, producing an array of double precision (64-bit) floating-point elements.
 * @param[in]	x	Pointer the minuend array of double precision (64-bit) floating-point elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of double precision (64-bit) floating-point elements to be subtracted.
 * @param[out]	difference	Pointer the array of double precision (64-bit) floating-point elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V64fV64f_V64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup yepCore
 * @defgroup yepCore_Multiply	Multiplication
 */
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 8-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 8-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 8-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uV8u_V8u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep8u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 8-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 8-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 16-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uV8u_V16u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two signed 8-bit integer arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 8-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of signed 8-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of signed 16-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8sV8s_V16s(const Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, Yep16s *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 16-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 16-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 16-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uV16u_V16u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 16-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 16-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 32-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uV16u_V32u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two signed 16-bit integer arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 16-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of signed 16-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of signed 32-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16sV16s_V32s(const Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, Yep32s *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 32-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 32-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 32-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uV32u_V32u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 32-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 32-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 64-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD K10</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uV32u_V64u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two signed 32-bit integer arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 32-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of signed 32-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of signed 64-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, AVX2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32sV32s_V64s(const Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep64s *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two unsigned 64-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 64-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 64-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 64-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64uV64u_V64u(const Yep64u *YEP_RESTRICT x, const Yep64u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two single precision (32-bit) floating-point arrays, producing an array of single precision (32-bit) floating-point elements.
 * @param[in]	x	Pointer the first array of single precision (32-bit) floating-point elements to be multiplied.
 * @param[in]	y	Pointer the second array of single precision (32-bit) floating-point elements to be multiplied.
 * @param[out]	product	Pointer the array of single precision (32-bit) floating-point elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32fV32f_V32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore_Multiply
 * @brief	Multiples corresponding elements in two double precision (64-bit) floating-point arrays, producing an array of double precision (64-bit) floating-point elements.
 * @param[in]	x	Pointer the first array of double precision (64-bit) floating-point elements to be multiplied.
 * @param[in]	y	Pointer the second array of double precision (64-bit) floating-point elements to be multiplied.
 * @param[out]	product	Pointer the array of double precision (64-bit) floating-point elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64fV64f_V64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uS8u_V8u(const Yep8u *YEP_RESTRICT x, Yep8u y, Yep8u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uS8u_V16u(const Yep8u *YEP_RESTRICT x, Yep8u y, Yep16u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8sS8s_V16s(const Yep8s *YEP_RESTRICT x, Yep8s y, Yep16s *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uS16u_V16u(const Yep16u *YEP_RESTRICT x, Yep16u y, Yep16u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uS16u_V32u(const Yep16u *YEP_RESTRICT x, Yep16u y, Yep32u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16sS16s_V32s(const Yep16s *YEP_RESTRICT x, Yep16s y, Yep32s *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uS32u_V32u(const Yep32u *YEP_RESTRICT x, Yep32u y, Yep32u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uS32u_V64u(const Yep32u *YEP_RESTRICT x, Yep32u y, Yep64u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32sS32s_V64s(const Yep32s *YEP_RESTRICT x, Yep32s y, Yep64s *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64uS64u_V64u(const Yep64u *YEP_RESTRICT x, Yep64u y, Yep64u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32fS32f_V32f(const Yep32f *YEP_RESTRICT x, Yep32f y, Yep32f *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64fS64f_V64f(const Yep64f *YEP_RESTRICT x, Yep64f y, Yep64f *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV8uV8u_IV8u(Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV16uV16u_IV16u(Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV32uV32u_IV32u(Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV64uV64u_IV64u(Yep64u *YEP_RESTRICT x, const Yep64u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV32fV32f_IV32f(Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV64fV64f_IV64f(Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV8uS8u_IV8u(Yep8u *YEP_RESTRICT x, Yep8u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV16uS16u_IV16u(Yep16u *YEP_RESTRICT x, Yep16u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV32uS32u_IV32u(Yep32u *YEP_RESTRICT x, Yep32u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV64uS64u_IV64u(Yep64u *YEP_RESTRICT x, Yep64u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV32fS32f_IV32f(Yep32f *YEP_RESTRICT x, Yep32f y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV64fS64f_IV64f(Yep64f *YEP_RESTRICT x, Yep64f y, YepSize length);
/**
 * @ingroup yepCore
 * @defgroup yepCore_Min	Minimum
 */
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of signed 8-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V8s_S8s(const Yep8s *YEP_RESTRICT v, Yep8s *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of unsigned 8-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V8u_S8u(const Yep8u *YEP_RESTRICT v, Yep8u *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of signed 16-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V16s_S16s(const Yep16s *YEP_RESTRICT v, Yep16s *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of unsigned 16-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V16u_S16u(const Yep16u *YEP_RESTRICT v, Yep16u *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of signed 32-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32s_S32s(const Yep32s *YEP_RESTRICT v, Yep32s *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of unsigned 32-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32u_S32u(const Yep32u *YEP_RESTRICT v, Yep32u *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of signed 64-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64s_S64s(const Yep64s *YEP_RESTRICT v, Yep64s *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of unsigned 64-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64u_S64u(const Yep64u *YEP_RESTRICT v, Yep64u *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of single precision (32-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32f_S32f(const Yep32f *YEP_RESTRICT v, Yep32f *YEP_RESTRICT minimum, YepSize length);
/**
 * @ingroup	yepCore_Min
 * @brief	Computes the minimum of double precision (64-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements whose minimum will be computed.
 * @param[out]	minimum	Pointer to the variable where the minimum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a minimum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a minimum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64f_S64f(const Yep64f *YEP_RESTRICT v, Yep64f *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V8sV8s_V8s(const Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, Yep8s *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V8uV8u_V8u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep8u *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V16sV16s_V16s(const Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, Yep16s *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V16uV16u_V16u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32sV32s_V32s(const Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep32s *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32uV32u_V32u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64sV32s_V64s(const Yep64s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep64s *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64uV32u_V64u(const Yep64u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32fV32f_V32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64fV64f_V64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V8sS8s_V8s(const Yep8s *YEP_RESTRICT x, Yep8s y, Yep8s *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V8uS8u_V8u(const Yep8u *YEP_RESTRICT x, Yep8u y, Yep8u *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V16sS16s_V16s(const Yep16s *YEP_RESTRICT x, Yep16s y, Yep16s *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V16uS16u_V16u(const Yep16u *YEP_RESTRICT x, Yep16u y, Yep16u *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32sS32s_V32s(const Yep32s *YEP_RESTRICT x, Yep32s y, Yep32s *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32uS32u_V32u(const Yep32u *YEP_RESTRICT x, Yep32u y, Yep32u *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64sS32s_V64s(const Yep64s *YEP_RESTRICT x, Yep32s y, Yep64s *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64uS32u_V64u(const Yep64u *YEP_RESTRICT x, Yep32u y, Yep64u *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V32fS32f_V32f(const Yep32f *YEP_RESTRICT x, Yep32f y, Yep32f *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_V64fS64f_V64f(const Yep64f *YEP_RESTRICT x, Yep64f y, Yep64f *YEP_RESTRICT minimum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV8sV8s_IV8s(Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV8uV8u_IV8u(Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV16sV16s_IV16s(Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV16uV16u_IV16u(Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV32sV32s_IV32s(Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV32uV32u_IV32u(Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV64sV32s_IV64s(Yep64s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV64uV32u_IV64u(Yep64u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV32fV32f_IV32f(Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV64fV64f_IV64f(Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV8sS8s_IV8s(Yep8s *YEP_RESTRICT x, Yep8s y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV8uS8u_IV8u(Yep8u *YEP_RESTRICT x, Yep8u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV16sS16s_IV16s(Yep16s *YEP_RESTRICT x, Yep16s y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV16uS16u_IV16u(Yep16u *YEP_RESTRICT x, Yep16u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV32sS32s_IV32s(Yep32s *YEP_RESTRICT x, Yep32s y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV32uS32u_IV32u(Yep32u *YEP_RESTRICT x, Yep32u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV64sS32s_IV64s(Yep64s *YEP_RESTRICT x, Yep32s y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV64uS32u_IV64u(Yep64u *YEP_RESTRICT x, Yep32u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV32fS32f_IV32f(Yep32f *YEP_RESTRICT x, Yep32f y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Min_IV64fS64f_IV64f(Yep64f *YEP_RESTRICT x, Yep64f y, YepSize length);
/**
 * @ingroup yepCore
 * @defgroup yepCore_Max	Maximum
 */
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of signed 8-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V8s_S8s(const Yep8s *YEP_RESTRICT v, Yep8s *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of unsigned 8-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V8u_S8u(const Yep8u *YEP_RESTRICT v, Yep8u *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of signed 16-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V16s_S16s(const Yep16s *YEP_RESTRICT v, Yep16s *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of unsigned 16-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V16u_S16u(const Yep16u *YEP_RESTRICT v, Yep16u *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of signed 32-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32s_S32s(const Yep32s *YEP_RESTRICT v, Yep32s *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of unsigned 32-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32u_S32u(const Yep32u *YEP_RESTRICT v, Yep32u *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of signed 64-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64s_S64s(const Yep64s *YEP_RESTRICT v, Yep64s *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of unsigned 64-bit integer array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64u_S64u(const Yep64u *YEP_RESTRICT v, Yep64u *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of single precision (32-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32f_S32f(const Yep32f *YEP_RESTRICT v, Yep32f *YEP_RESTRICT maximum, YepSize length);
/**
 * @ingroup	yepCore_Max
 * @brief	Computes the maximum of double precision (64-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements whose maximum will be computed.
 * @param[out]	maximum	Pointer to the variable where the maximum will be stored.
 * @param[in]	length	The length of the array specified by @a v. Must be non-zero.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a maximum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a maximum argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64f_S64f(const Yep64f *YEP_RESTRICT v, Yep64f *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V8sV8s_V8s(const Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, Yep8s *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V8uV8u_V8u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep8u *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V16sV16s_V16s(const Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, Yep16s *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V16uV16u_V16u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32sV32s_V32s(const Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep32s *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32uV32u_V32u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64sV32s_V64s(const Yep64s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep64s *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64uV32u_V64u(const Yep64u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32fV32f_V32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64fV64f_V64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V8sS8s_V8s(const Yep8s *YEP_RESTRICT x, Yep8s y, Yep8s *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V8uS8u_V8u(const Yep8u *YEP_RESTRICT x, Yep8u y, Yep8u *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V16sS16s_V16s(const Yep16s *YEP_RESTRICT x, Yep16s y, Yep16s *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V16uS16u_V16u(const Yep16u *YEP_RESTRICT x, Yep16u y, Yep16u *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32sS32s_V32s(const Yep32s *YEP_RESTRICT x, Yep32s y, Yep32s *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32uS32u_V32u(const Yep32u *YEP_RESTRICT x, Yep32u y, Yep32u *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64sS32s_V64s(const Yep64s *YEP_RESTRICT x, Yep32s y, Yep64s *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64uS32u_V64u(const Yep64u *YEP_RESTRICT x, Yep32u y, Yep64u *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V32fS32f_V32f(const Yep32f *YEP_RESTRICT x, Yep32f y, Yep32f *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_V64fS64f_V64f(const Yep64f *YEP_RESTRICT x, Yep64f y, Yep64f *YEP_RESTRICT maximum, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV8sV8s_IV8s(Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV8uV8u_IV8u(Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV16sV16s_IV16s(Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV16uV16u_IV16u(Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV32sV32s_IV32s(Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV32uV32u_IV32u(Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV64sV32s_IV64s(Yep64s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV64uV32u_IV64u(Yep64u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV32fV32f_IV32f(Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV64fV64f_IV64f(Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV8sS8s_IV8s(Yep8s *YEP_RESTRICT x, Yep8s y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV8uS8u_IV8u(Yep8u *YEP_RESTRICT x, Yep8u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV16sS16s_IV16s(Yep16s *YEP_RESTRICT x, Yep16s y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV16uS16u_IV16u(Yep16u *YEP_RESTRICT x, Yep16u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV32sS32s_IV32s(Yep32s *YEP_RESTRICT x, Yep32s y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV32uS32u_IV32u(Yep32u *YEP_RESTRICT x, Yep32u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV64sS32s_IV64s(Yep64s *YEP_RESTRICT x, Yep32s y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV64uS32u_IV64u(Yep64u *YEP_RESTRICT x, Yep32u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV32fS32f_IV32f(Yep32f *YEP_RESTRICT x, Yep32f y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Max_IV64fS64f_IV64f(Yep64f *YEP_RESTRICT x, Yep64f y, YepSize length);
/**
 * @ingroup yepCore
 * @defgroup yepCore_Sum	Sum
 */
/**
 * @ingroup	yepCore_Sum
 * @brief	Computes the sum of single precision (32-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements which will be summed up.
 * @param[out]	sum	Pointer to the variable where the sum will be stored.
 * @param[in]	length	The length of the array specified by @a v. If @a length is zero, the computed sum will be 0.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Sum_V32f_S32f(const Yep32f *YEP_RESTRICT v, Yep32f *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore_Sum
 * @brief	Computes the sum of double precision (64-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements which will be summed up.
 * @param[out]	sum	Pointer to the variable where the sum will be stored.
 * @param[in]	length	The length of the array specified by @a v. If @a length is zero, the computed sum will be 0.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Sum_V64f_S64f(const Yep64f *YEP_RESTRICT v, Yep64f *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup yepCore
 * @defgroup yepCore_SumAbs	Sum of absolute values
 */
/**
 * @ingroup	yepCore_SumAbs
 * @brief	Computes the sum of absolute values of single precision (32-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements whose absolute values will be summed up.
 * @param[out]	sumAbs	Pointer to the variable where the sum of absolute values will be stored.
 * @param[in]	length	The length of the array specified by @a v. If @a length is zero, the computed sum will be 0.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a sumAbs argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a sumAbs argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE2, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_SumAbs_V32f_S32f(const Yep32f *YEP_RESTRICT v, Yep32f *YEP_RESTRICT sumAbs, YepSize length);
/**
 * @ingroup	yepCore_SumAbs
 * @brief	Computes the sum of absolute values of double precision (64-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements whose absolute values will be summed up.
 * @param[out]	sumAbs	Pointer to the variable where the sum of absolute values will be stored.
 * @param[in]	length	The length of the array specified by @a v. If @a length is zero, the computed sum will be 0.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a sumAbs argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a sumAbs argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_SumAbs_V64f_S64f(const Yep64f *YEP_RESTRICT v, Yep64f *YEP_RESTRICT sumAbs, YepSize length);
/**
 * @ingroup yepCore
 * @defgroup yepCore_SumSquares	Sum of squares (squared L2 norm)
 */
/**
 * @ingroup	yepCore_SumSquares
 * @brief	Computes the sum of squares of single precision (32-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements which will be squared (without write-back) and summed up.
 * @param[out]	sumSquares	Pointer to the variable where the sum of squares will be stored.
 * @param[in]	length	The length of the array specified by @a v. If @a length is zero, the computed sum of squares will be 0.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a sumSquares argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a sumSquares argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, FMA3</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX, FMA4</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_SumSquares_V32f_S32f(const Yep32f *YEP_RESTRICT v, Yep32f *YEP_RESTRICT sumSquares, YepSize length);
/**
 * @ingroup	yepCore_SumSquares
 * @brief	Computes the sum of squares of double precision (64-bit) floating-point array elements.
 * @param[in]	v	Pointer to the array of elements which will be squared (without write-back) and summed up.
 * @param[out]	sumSquares	Pointer to the variable where the sum of squares will be stored.
 * @param[in]	length	The length of the array specified by @a v. If @a length is zero, the computed sum of squares will be 0.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a sumSquares argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a sumSquares argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, FMA3</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX, FMA4</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_SumSquares_V64f_S64f(const Yep64f *YEP_RESTRICT v, Yep64f *YEP_RESTRICT sumSquares, YepSize length);
/**
 * @ingroup yepCore
 * @defgroup yepCore_DotProduct	Dot product
 */
/**
 * @ingroup	yepCore_DotProduct
 * @brief	Computes the dot product of two vectors of single precision (32-bit) floating-point elements.
 * @param[in]	x	Pointer to the first vector of elements.
 * @param[in]	y	Pointer to the second vector of elements.
 * @param[out]	dotProduct	Pointer to the variable where the dot product value will be stored.
 * @param[in]	length	The length of the vectors specified by @a x and @a y.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a dotProduct argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a dotProduct argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, FMA3</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX, FMA4</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_DotProduct_V32fV32f_S32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT dotProduct, YepSize length);
/**
 * @ingroup	yepCore_DotProduct
 * @brief	Computes the dot product of two vectors of double precision (64-bit) floating-point elements.
 * @param[in]	x	Pointer to the first vector of elements.
 * @param[in]	y	Pointer to the second vector of elements.
 * @param[out]	dotProduct	Pointer to the variable where the dot product value will be stored.
 * @param[in]	length	The length of the vectors specified by @a x and @a y.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a dotProduct argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a dotProduct argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, FMA3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Bonnell</td><td>SSE, SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX, FMA4</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_DotProduct_V64fV64f_S64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT dotProduct, YepSize length);
#ifdef __cplusplus
	} // extern "C"
#endif